<?php

/**
 * @noinspection PhpUnused
 * @noinspection PhpUnusedParameterInspection
 */

namespace TelegramApi;

use TelegramApi\Types\Chat;
use TelegramApi\Types\ChatMember;
use TelegramApi\Types\File;
use TelegramApi\Types\ForceReply;
use TelegramApi\Types\GameHighScore;
use TelegramApi\Types\InlineKeyboardMarkup;
use TelegramApi\Types\InlineQueryResult;
use TelegramApi\Types\InputFile;
use TelegramApi\Types\Message;
use TelegramApi\Types\ReplyKeyboardMarkup;
use TelegramApi\Types\ReplyKeyboardRemove;
use TelegramApi\Types\Type;
use TelegramApi\Types\Update;
use TelegramApi\Types\User;
use TelegramApi\Types\UserProfilePhotos;
use TelegramApi\Types\WebhookInfo;

trait API
{
	abstract public function sendRequest(string $method, array $args): mixed;


	/**
	 * Use this method to receive incoming updates using long polling (wiki). An Array of Update objects is returned.
	 * @param Type|null $parameters Description
	 * @param int|null $offset Identifier of the first update to be returned. Must be greater by one than the highest among the identifiers of previously received updates. By default, updates starting with the earliest unconfirmed update are returned. An update is considered confirmed as soon as getUpdates is called with an offset higher than its update_id. The negative offset can be specified to retrieve updates starting from -offset update from the end of the updates queue. All previous updates will forgotten.
	 * @param int|null $limit Limits the number of updates to be retrieved. Values between 1—100 are accepted. Defaults to 100.
	 * @param int|null $timeout Timeout in seconds for long polling. Defaults to 0, i.e. usual short polling. Should be positive, short polling should be used for testing purposes only.
	 * @return Array<Update>
	 */
	public function getUpdates(
		?Type $parameters = null,
		?int $offset = null,
		?int $limit = 100,
		?int $timeout = null,
	): array
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to specify a url and receive incoming updates via an outgoing webhook. Whenever there is an update for the bot, we will send an HTTPS POST request to the specified url, containing a JSON-serialized Update. In case of an unsuccessful request, we will give up after a reasonable amount of attempts.
	 * If you'd like to make sure that the Webhook request comes from Telegram, we recommend using a secret path in the URL, e.g. https://www.example.com/<token>. Since nobody else knows your bot‘s token, you can be pretty sure it’s us.
	 * @param Type|null $parameters Description
	 * @param string|null $url HTTPS url to send updates to. Use an empty string to remove webhook integration
	 * @param InputFile|null $certificate Upload your public key certificate so that the root certificate in use can be checked. See our self-signed guide for details.
	 */
	public function setWebhook(?Type $parameters = null, ?string $url = null, ?InputFile $certificate = null)
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get current webhook status. Requires no parameters. On success, returns a WebhookInfo object. If the bot is using getUpdates, will return an object with the url field empty.
	 * @return WebhookInfo
	 */
	public function getWebhookInfo(): WebhookInfo
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * A simple method for testing your bot's auth token. Requires no parameters. Returns basic information about the bot in form of a User object.
	 * @return User
	 */
	public function getMe(): User
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send text messages. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param string $text Text of the message to be sent
	 * @param Type|null $parameters Description
	 * @param string|null $parseMode Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in your bot's message.
	 * @param bool|null $disableWebPagePreview Disables link previews for links in this message
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendMessage(
		int|string $chatId,
		string $text,
		?Type $parameters = null,
		?string $parseMode = null,
		?bool $disableWebPagePreview = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to forward messages of any kind. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param int|string $fromChatId Unique identifier for the chat where the original message was sent (or channel username in the format @channelusername)
	 * @param int $messageId Message identifier in the chat specified in from_chat_id
	 * @param Type|null $parameters Description
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @return Message
	 */
	public function forwardMessage(
		int|string $chatId,
		int|string $fromChatId,
		int $messageId,
		?Type $parameters = null,
		?bool $disableNotification = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send photos. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $photo Photo to send. Pass a file_id as String to send a photo that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get a photo from the Internet, or upload a new photo using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param string|null $caption Photo caption (may also be used when resending photos by file_id), 0-200 characters
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendPhoto(
		int|string $chatId,
		InputFile|string $photo,
		?Type $parameters = null,
		?string $caption = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send audio files, if you want Telegram clients to display them in the music player. Your audio must be in the .mp3 format. On success, the sent Message is returned. Bots can currently send audio files of up to 50 MB in size, this limit may be changed in the future.
	 * For sending voice messages, use the sendVoice method instead.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $audio Audio file to send. Pass a file_id as String to send an audio file that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get an audio file from the Internet, or upload a new one using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param string|null $caption Audio caption, 0-200 characters
	 * @param int|null $duration Duration of the audio in seconds
	 * @param string|null $performer Performer
	 * @param string|null $title Track name
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendAudio(
		int|string $chatId,
		InputFile|string $audio,
		?Type $parameters = null,
		?string $caption = null,
		?int $duration = null,
		?string $performer = null,
		?string $title = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send general files. On success, the sent Message is returned. Bots can currently send files of any type of up to 50 MB in size, this limit may be changed in the future.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $document File to send. Pass a file_id as String to send a file that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get a file from the Internet, or upload a new one using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param string|null $caption Document caption (may also be used when resending documents by file_id), 0-200 characters
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendDocument(
		int|string $chatId,
		InputFile|string $document,
		?Type $parameters = null,
		?string $caption = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send .webp stickers. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $sticker Sticker to send. Pass a file_id as String to send a file that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get a .webp file from the Internet, or upload a new one using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendSticker(
		int|string $chatId,
		InputFile|string $sticker,
		?Type $parameters = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send video files, Telegram clients support mp4 videos (other formats may be sent as Document). On success, the sent Message is returned. Bots can currently send video files of up to 50 MB in size, this limit may be changed in the future.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $video Video to send. Pass a file_id as String to send a video that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get a video from the Internet, or upload a new video using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param int|null $duration Duration of sent video in seconds
	 * @param int|null $width Video width
	 * @param int|null $height Video height
	 * @param string|null $caption Video caption (may also be used when resending videos by file_id), 0-200 characters
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendVideo(
		int|string $chatId,
		InputFile|string $video,
		?Type $parameters = null,
		?int $duration = null,
		?int $width = null,
		?int $height = null,
		?string $caption = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send audio files, if you want Telegram clients to display the file as a playable voice message. For this to work, your audio must be in an .ogg file encoded with OPUS (other formats may be sent as Audio or Document). On success, the sent Message is returned. Bots can currently send voice messages of up to 50 MB in size, this limit may be changed in the future.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param InputFile|string $voice Audio file to send. Pass a file_id as String to send a file that exists on the Telegram servers (recommended), pass an HTTP URL as a String for Telegram to get a file from the Internet, or upload a new one using multipart/form-data. More info on Sending Files »
	 * @param Type|null $parameters Description
	 * @param string|null $caption Voice message caption, 0-200 characters
	 * @param int|null $duration Duration of the voice message in seconds
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendVoice(
		int|string $chatId,
		InputFile|string $voice,
		?Type $parameters = null,
		?string $caption = null,
		?int $duration = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send point on the map. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param float $latitude Latitude of location
	 * @param float $longitude Longitude of location
	 * @param Type|null $parameters Description
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendLocation(
		int|string $chatId,
		float $latitude,
		float $longitude,
		?Type $parameters = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send information about a venue. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param float $latitude Latitude of the venue
	 * @param float $longitude Longitude of the venue
	 * @param string $title Name of the venue
	 * @param string $address Address of the venue
	 * @param Type|null $parameters Description
	 * @param string|null $foursquareId Foursquare identifier of the venue
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove reply keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendVenue(
		int|string $chatId,
		float $latitude,
		float $longitude,
		string $title,
		string $address,
		?Type $parameters = null,
		?string $foursquareId = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send phone contacts. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param string $phoneNumber Contact's phone number
	 * @param string $firstName Contact's first name
	 * @param Type|null $parameters Description
	 * @param string|null $lastName Contact's last name
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup Additional interface options. A JSON-serialized object for an inline keyboard, custom reply keyboard, instructions to remove keyboard or to force a reply from the user.
	 * @return Message
	 */
	public function sendContact(
		int|string $chatId,
		string $phoneNumber,
		string $firstName,
		?Type $parameters = null,
		?string $lastName = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		InlineKeyboardMarkup|ReplyKeyboardMarkup|ReplyKeyboardRemove|ForceReply|null $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method when you need to tell the user that something is happening on the bot's side. The status is set for 5 seconds or less (when a message arrives from your bot, Telegram clients clear its typing status). Returns True on success.
	 * We only recommend using this method when a response from the bot will take a noticeable amount of time to arrive.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param string $action Type of action to broadcast. Choose one, depending on what the user is about to receive: typing for text messages, upload_photo for photos, record_video or upload_video for videos, record_audio or upload_audio for audio files, upload_document for general files, find_location for location data.
	 * @param Type|null $parameters Description
	 * @return bool
	 */
	public function sendChatAction(int|string $chatId, string $action, ?Type $parameters = null): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get a list of profile pictures for a user. Returns a UserProfilePhotos object.
	 * @param int $userId Unique identifier of the target user
	 * @param Type|null $parameters Description
	 * @param int|null $offset Sequential number of the first photo to be returned. By default, all photos are returned.
	 * @param int|null $limit Limits the number of photos to be retrieved. Values between 1—100 are accepted. Defaults to 100.
	 * @return UserProfilePhotos
	 */
	public function getUserProfilePhotos(
		int $userId,
		?Type $parameters = null,
		?int $offset = null,
		?int $limit = 100,
	): UserProfilePhotos
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get basic info about a file and prepare it for downloading. For the moment, bots can download files of up to 20MB in size. On success, a File object is returned. The file can then be downloaded via the link https://api.telegram.org/file/bot<token>/<file_path>, where <file_path> is taken from the response. It is guaranteed that the link will be valid for at least 1 hour. When the link expires, a new one can be requested by calling getFile again.
	 * @param string $fileId File identifier to get info about
	 * @param Type|null $parameters Description
	 * @return File
	 */
	public function getFile(string $fileId, ?Type $parameters = null): File
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to kick a user from a group or a supergroup. In the case of supergroups, the user will not be able to return to the group on their own using invite links, etc., unless unbanned first. The bot must be an administrator in the group for this to work. Returns True on success.
	 * @param int|string $chatId Unique identifier for the target group or username of the target supergroup (in the format @supergroupusername)
	 * @param int $userId Unique identifier of the target user
	 * @param Type|null $parameters Description
	 * @return bool
	 */
	public function kickChatMember(int|string $chatId, int $userId, ?Type $parameters = null): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method for your bot to leave a group, supergroup or channel. Returns True on success.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
	 * @param Type|null $parameters Description
	 * @return bool
	 */
	public function leaveChat(int|string $chatId, ?Type $parameters = null): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to unban a previously kicked user in a supergroup. The user will not return to the group automatically, but will be able to join via link, etc. The bot must be an administrator in the group for this to work. Returns True on success.
	 * @param int|string $chatId Unique identifier for the target group or username of the target supergroup (in the format @supergroupusername)
	 * @param int $userId Unique identifier of the target user
	 * @param Type|null $parameters Description
	 * @return bool
	 */
	public function unbanChatMember(int|string $chatId, int $userId, ?Type $parameters = null): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get up to date information about the chat (current name of the user for one-on-one conversations, current username of a user, group or channel, etc.). Returns a Chat object on success.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
	 * @param Type|null $parameters Description
	 * @return Chat
	 */
	public function getChat(int|string $chatId, ?Type $parameters = null): Chat
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get a list of administrators in a chat. On success, returns an Array of ChatMember objects that contains information about all chat administrators except other bots. If the chat is a group or a supergroup and no administrators were appointed, only the creator will be returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
	 * @param Type|null $parameters Description
	 * @return Array<ChatMember>
	 */
	public function getChatAdministrators(int|string $chatId, ?Type $parameters = null): array
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get the number of members in a chat. Returns Int on success.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
	 * @param Type|null $parameters Description
	 * @return int
	 */
	public function getChatMembersCount(int|string $chatId, ?Type $parameters = null): int
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get information about a member of a chat. Returns a ChatMember object on success.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target supergroup or channel (in the format @channelusername)
	 * @param int $userId Unique identifier of the target user
	 * @param Type|null $parameters Description
	 * @return ChatMember
	 */
	public function getChatMember(int|string $chatId, int $userId, ?Type $parameters = null): ChatMember
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send answers to callback queries sent from inline keyboards. The answer will be displayed to the user as a notification at the top of the chat screen or as an alert. On success, True is returned.
	 * @param string $callbackQueryId Unique identifier for the query to be answered
	 * @param Type|null $parameters Description
	 * @param string|null $text Text of the notification. If not specified, nothing will be shown to the user, 0-200 characters
	 * @param bool|null $showAlert If true, an alert will be shown by the client instead of a notification at the top of the chat screen. Defaults to false.
	 * @param string|null $url URL that will be opened by the user's client. If you have created a Game and accepted the conditions via @Botfather, specify the URL that opens your game – note that this will only work if the query comes from a callback_game button.Otherwise, you may use links like telegram.me/your_bot?start=XXXX that open your bot with a parameter.
	 * @param int|null $cacheTime The maximum amount of time in seconds that the result of the callback query may be cached client-side. Telegram apps will support caching starting in version 3.14. Defaults to 0.
	 * @return bool
	 */
	public function answerCallbackQuery(
		string $callbackQueryId,
		?Type $parameters = null,
		?string $text = null,
		?bool $showAlert = null,
		?string $url = null,
		?int $cacheTime = null,
	): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to edit text and game messages sent by the bot or via the bot (for inline bots). On success, if edited message is sent by the bot, the edited Message is returned, otherwise True is returned.
	 * @param string $text New text of the message
	 * @param Type|null $parameters Description
	 * @param int|string|null $chatId Required if inline_message_id is not specified. Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param int|null $messageId Required if inline_message_id is not specified. Identifier of the sent message
	 * @param string|null $inlineMessageId Required if chat_id and message_id are not specified. Identifier of the inline message
	 * @param string|null $parseMode Send Markdown or HTML, if you want Telegram apps to show bold, italic, fixed-width text or inline URLs in your bot's message.
	 * @param bool|null $disableWebPagePreview Disables link previews for links in this message
	 * @param InlineKeyboardMarkup|null $replyMarkup A JSON-serialized object for an inline keyboard.
	 * @return Message|bool
	 */
	public function editMessageText(
		string $text,
		?Type $parameters = null,
		int|string|null $chatId = null,
		?int $messageId = null,
		?string $inlineMessageId = null,
		?string $parseMode = null,
		?bool $disableWebPagePreview = null,
		?InlineKeyboardMarkup $replyMarkup = null,
	): Message|bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to edit captions of messages sent by the bot or via the bot (for inline bots). On success, if edited message is sent by the bot, the edited Message is returned, otherwise True is returned.
	 * @param Type|null $parameters Description
	 * @param int|string|null $chatId Required if inline_message_id is not specified. Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param int|null $messageId Required if inline_message_id is not specified. Identifier of the sent message
	 * @param string|null $inlineMessageId Required if chat_id and message_id are not specified. Identifier of the inline message
	 * @param string|null $caption New caption of the message
	 * @param InlineKeyboardMarkup|null $replyMarkup A JSON-serialized object for an inline keyboard.
	 * @return Message|bool
	 */
	public function editMessageCaption(
		?Type $parameters = null,
		int|string|null $chatId = null,
		?int $messageId = null,
		?string $inlineMessageId = null,
		?string $caption = null,
		?InlineKeyboardMarkup $replyMarkup = null,
	): Message|bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to edit only the reply markup of messages sent by the bot or via the bot (for inline bots).  On success, if edited message is sent by the bot, the edited Message is returned, otherwise True is returned.
	 * @param Type|null $parameters Description
	 * @param int|string|null $chatId Required if inline_message_id is not specified. Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param int|null $messageId Required if inline_message_id is not specified. Identifier of the sent message
	 * @param string|null $inlineMessageId Required if chat_id and message_id are not specified. Identifier of the inline message
	 * @param InlineKeyboardMarkup|null $replyMarkup A JSON-serialized object for an inline keyboard.
	 * @return Message|bool
	 */
	public function editMessageReplyMarkup(
		?Type $parameters = null,
		int|string|null $chatId = null,
		?int $messageId = null,
		?string $inlineMessageId = null,
		?InlineKeyboardMarkup $replyMarkup = null,
	): Message|bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send answers to an inline query. On success, True is returned.No more than 50 results per query are allowed.
	 * @param string $inlineQueryId Unique identifier for the answered query
	 * @param Array<InlineQueryResult> $results A JSON-serialized array of results for the inline query
	 * @param Type|null $parameters Description
	 * @param int|null $cacheTime The maximum amount of time in seconds that the result of the inline query may be cached on the server. Defaults to 300.
	 * @param bool|null $isPersonal Pass True, if results may be cached on the server side only for the user that sent the query. By default, results may be returned to any user who sends the same query
	 * @param string|null $nextOffset Pass the offset that a client should send in the next query with the same text to receive more results. Pass an empty string if there are no more results or if you don‘t support pagination. Offset length can’t exceed 64 bytes.
	 * @param string|null $switchPmText If passed, clients will display a button with specified text that switches the user to a private chat with the bot and sends the bot a start message with the parameter switch_pm_parameter
	 * @param string|null $switchPmParameter Parameter for the start message sent to the bot when user presses the switch buttonExample: An inline bot that sends YouTube videos can ask the user to connect the bot to their YouTube account to adapt search results accordingly. To do this, it displays a ‘Connect your YouTube account’ button above the results, or even before showing any. The user presses the button, switches to a private chat with the bot and, in doing so, passes a start parameter that instructs the bot to return an oauth link. Once done, the bot can offer a switch_inline button so that the user can easily return to the chat where they wanted to use the bot's inline capabilities.
	 * @return bool
	 */
	public function answerInlineQuery(
		string $inlineQueryId,
		array $results,
		?Type $parameters = null,
		?int $cacheTime = 300,
		?bool $isPersonal = null,
		?string $nextOffset = null,
		?string $switchPmText = null,
		?string $switchPmParameter = null,
	): bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to send a game. On success, the sent Message is returned.
	 * @param int|string $chatId Unique identifier for the target chat or username of the target channel (in the format @channelusername)
	 * @param string $gameShortName Short name of the game, serves as the unique identifier for the game. Set up your games via Botfather.
	 * @param Type|null $parameters Description
	 * @param bool|null $disableNotification Sends the message silently. iOS users will not receive a notification, Android users will receive a notification with no sound.
	 * @param int|null $replyToMessageId If the message is a reply, ID of the original message
	 * @param InlineKeyboardMarkup|null $replyMarkup A JSON-serialized object for an inline keyboard. If empty, one ‘Play game_title’ button will be shown. If not empty, the first button must launch the game.
	 * @return Message
	 */
	public function sendGame(
		int|string $chatId,
		string $gameShortName,
		?Type $parameters = null,
		?bool $disableNotification = null,
		?int $replyToMessageId = null,
		?InlineKeyboardMarkup $replyMarkup = null,
	): Message
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to set the score of the specified user in a game. On success, if the message was sent by the bot, returns the edited Message, otherwise returns True. Returns an error, if the new score is not greater than the user's current score in the chat and force is False.
	 * @param int $userId User identifier
	 * @param int $score New score, must be non-negative
	 * @param Type|null $parameters Description
	 * @param bool|null $force Pass True, if the high score is allowed to decrease. This can be useful when fixing mistakes or banning cheaters
	 * @param bool|null $disableEditMessage Pass True, if the game message should not be automatically edited to include the current scoreboard
	 * @param int|string|null $chatId Required if inline_message_id is not specified. Unique identifier for the target chat (or username of the target channel in the format @channelusername)
	 * @param int|null $messageId Required if inline_message_id is not specified. Identifier of the sent message
	 * @param string|null $inlineMessageId Required if chat_id and message_id are not specified. Identifier of the inline message
	 * @return Message|bool
	 */
	public function setGameScore(
		int $userId,
		int $score,
		?Type $parameters = null,
		?bool $force = null,
		?bool $disableEditMessage = null,
		int|string|null $chatId = null,
		?int $messageId = null,
		?string $inlineMessageId = null,
	): Message|bool
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}


	/**
	 * Use this method to get data for high score tables. Will return the score of the specified user and several of his neighbors in a game. On success, returns an Array of GameHighScore objects.
	 * @param int $userId Target user id
	 * @param Type|null $parameters Description
	 * @param int|string|null $chatId Required if inline_message_id is not specified. Unique identifier for the target chat (or username of the target channel in the format @channelusername)
	 * @param int|null $messageId Required if inline_message_id is not specified. Identifier of the sent message
	 * @param string|null $inlineMessageId Required if chat_id and message_id are not specified. Identifier of the inline message
	 * @return Array<GameHighScore>
	 */
	public function getGameHighScores(
		int $userId,
		?Type $parameters = null,
		int|string|null $chatId = null,
		?int $messageId = null,
		?string $inlineMessageId = null,
	): array
	{
		$args = get_defined_vars();
		return $this->sendRequest(__FUNCTION__, $args);
	}
}
